function [B,insulin,C_calcium] = Automaton_Function(A,B,gluc_soll,kali_soll,StartStimu,t,n_curr,n_nom)
%% This is the automaton function. 
%
% Inputs:
% A := Matrix that contains the information about position and status of
%      granules of last iteration step
% B := Matrix that contains only position on actin net
% gluc_soll/kali_soll := values of externally inserted concentrations of
%                        glucose/kalium
% StartSimu := variable that holds value of stimulation of last iteration
%              step
% t := time variable for ODE
% n_curr := current number of granules
% n_nom := number of granules at beginning


%% === System variables ===
[x,y,z] = size(A(:,:,:,1)); % determine size of whole domain, (for speedup this can be moved outside of automaton function)
insulin = 0; % evaluation variable; do not change; holds insulin that is emitted each step

%% === ODE for time dependend change of stimulations ===
if t == 0
    C_calcium = StartStimu;
else
    [~,C_calcium] = ode45(@(t,Stimulation)(Stimulation_ODE(Stimulation,gluc_soll,kali_soll)),[0 t],StartStimu);
    C_calcium = C_calcium(end);
end  

%% === Parameters to set ===
ripe   = 15;   % this variable determines when granule is ripe and therefore able to fuse
a_life = 1;    % factor ripeness
a_2    = 1/2;  % factor change movement from 2 to directed
a_23   = 1;    % factor change movement from 2 to 3
a_3    = 1;    % factor change movement from 3 to 2
a_5_10 = 2.8;    % factor change movement from directed to 2
a_act  = 1/3;  % factor go through actin (actin remodeling)
a_11   = 1;    % factor change movement from 11 to 2

%% === Stimulation Parameters ===== 
p_life = a_life*C_calcium;
p_2    = a_2*C_calcium;
p_23   = a_23*C_calcium;
p_3    = a_3*C_calcium;
p_5_10 = a_5_10*C_calcium;
p_act  = a_act*C_calcium;
p_11   = a_11*C_calcium;

%% === generation of new granules within the domain === %%

for i = ceil(0.30*x):floor(0.70*x)         % creation of new granules only
    for j = ceil(0.30*y):floor(0.70*y)     % in the inner part of the modeled
        for k = ceil(0.30*z):floor(0.70*z) % Beta Cell
            
            rel_dev = (n_nom-n_curr)/n_nom;   % determine relative deviation of granules
            if B(i,j,k,1) == 0 && (rel_dev > rand)
                if rand < 0.88
                    B(i,j,k,1) = 2;             % creation of diffusive moving granule
                    if rand < 0.05
                        B(i,j,k,2) = ripe+1;         % chance of 5% of being Readily Releasable
                    else
                        B(i,j,k,2) = randi([0 ripe]);  % otherwise ripeness value between 0 and time
                    end
                else
                    B(i,j,k,1) = randi([5 10]); % creation of directed moving granule
                    if rand < 0.05
                        B(i,j,k,2) = ripe+1;         % chance of 5% of being Readily Releasable
                    else
                        B(i,j,k,2) = randi([0 ripe]);  % otherwise ripeness value between 0 and time
                    end
                end
            end
            
        end
    end
end

%% ================= Calculation of granule movement =================== %%

% === Movement inside the domain/cell === %
for i=1:x
    for j=1:y
        for k=1:z 
            
            flag = 0;
            elem_state = A(i,j,k,1); % state of the element, e.g. fluid,actin or granule
            
            
                % === Fluid ===============================================
            if elem_state == 0 
                    continue
                
                % === Actin ===============================================
            elseif elem_state == 1
                    continue

%%%========== Calculation of diffusive moving Granules =================%%%
            elseif elem_state ==  2 
                    a = randi([-1 1]);
                    b = randi([-1 1]);
                    c = randi([-1 1]);
                    
                    try
                        if B(i+a,j+b,k+c,1) == 0  && A(i+a,j+b,k+c,1) == 0 %checking if matrix element is not occupied
                          if i+a==1 || i+a==x || j+b==1 || j+b==y || k+c==1 || k+c==z
                                B(i,j,k,1) = 11; %docking of granule to membrane
                                B(i,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k,2) = A(i,j,k,2) + 1;
                                end
                                
                          elseif rand > p_2 %calculation if granule should change current movement to directed movement depending on external stimulation
                                B(i+a,j+b,k+c,1) = 2;
                                B(i+a,j+b,k+c,2) = A(i,j,k,2);
                                if rand < p_life  %checking if ripeness should be increased due to external stimulation
                                    B(i+a,j+b,k+c,2) = A(i,j,k,2) + 1;
                                end
                            else
                                B(i+a,j+b,k+c,1) = randi([5 10]);
                                B(i+a,j+b,k+c,2) = A(i,j,k,2);
                                if rand < p_life %checking if ripeness should be increased due to external stimulation
                                    B(i+a,j+b,k+c,2) = A(i,j,k,2) + 1;
                                end
                          end
                            
                        elseif B(i+a,j+b,k+c,1) == 1 && rand > p_23
                            B(i,j,k,1) = 3; %Granule is attached to actin net
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life     %checking if ripeness should be increased due to external stimulation
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        else
                            B(i,j,k,1) = 2; %Granule does not move and does not change its movement state
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life    %%checking if ripeness should be increased due to external stimulation
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                    catch
                        if rand < C_calcium  && A(i,j,k,2) > ripe
                            B(i,j,k,1) = 4; 
                        else
                            B(i,j,k,1) = 2;
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life     %checking if ripeness should be increased due to external stimulation
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                    end
                    continue
                 
                    
%%%========== Calculation of to actin attached Granules ================%%%
            elseif elem_state == 3
                    if rand < p_3
                        B(i,j,k,1) = 2; %de-attachement of actin
                        B(i,j,k,2) = A(i,j,k,2);
                        if rand < p_life    
                            B(i,j,k,2) = A(i,j,k,2) + 1;
                        end
                    else
                        a = randi([-1 1]);
                        b = randi([-1 1]);
                        c = randi([-1 1]);

                        %checking if actin is next to newly calculated
                        %position so granules keeps attached:
                        try
                            if A(i+a+1,j+b,k+c,1) == 1 || A(i+a-1,j+b,k+c,1) == 1
                                flag = 1;                  
                            end
                            if A(i+a,j+b+1,k+c,1) == 1 || A(i+a,j+b-1,k+c,1) == 1
                                flag = 1;                  
                            end
                            if A(i+a,j+b,k+c+1,1) == 1 || A(i+a,j+b,k+c-1,1) == 1
                                flag = 1;                  
                            end
                        catch
                            flag = 0;
                        end
                        
                        try
                            if B(i+a,j+b,k+c,1) == 0 && flag == 1 && A(i+a,j+b,k+c,1) == 0
                                B(i+a,j+b,k+c,1) = 3; % Granule keeps attached
                                B(i+a,j+b,k+c,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i+a,j+b,k+c,2) = A(i,j,k,2) + 1;
                                end
                            else
                                B(i,j,k,1) = 3; % Granule keeps current position in this iteration step
                                B(i,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k,2) = A(i,j,k,2) + 1;
                                end
                            end
                        catch
                            B(i,j,k,1) = 3; % Granule keeps current position in this iteration step
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                    end
                    continue
                          
                    
%%%======================= Fusion of Granules ==========================%%%   
            elseif elem_state == 4 
                	B(i,j,k,1) = 0;
                    B(i,j,k,2) = 0;
                    insulin = insulin + 1; % increase of emitted insulin
                    
                    
%%%=========== calculation of directed movement along x-axis ===========%%%
            elseif elem_state == 5 || elem_state == 6
                    a = 0;
                    % checking if movement in neg. or pos. direction:
                    if elem_state == 5
                        Weg = 1; 
                    elseif elem_state == 6
                        Weg = -1;
                    end
                    
                    try
                          
                        if B(i+Weg,j,k,1) == 0 && A(i+Weg,j,k,1) == 0   %checking if newly calculated position is not occoupied
                            if i+Weg==1 || i+Weg==x
                                B(i,j,k,1) = 11; %attachement of granule to membrane
                                B(i,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k,2) = A(i,j,k,2) + 1;
                                end
                              
                            elseif rand < p_5_10 %calculating if directed movement should be changed to diffusive movement due to external stimulation
                                B(i+Weg,j,k,1) = elem_state;
                                B(i+Weg,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i+Weg,j,k,2) = A(i,j,k,2) + 1;
                                end
                            else 
                                B(i+Weg,j,k,1) = 2;
                                B(i+Weg,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i+Weg,j,k,2) = A(i,j,k,2) + 1;
                                end
                            end
                            
                        % Implementaion of actin-remodeling:
                        elseif A(i+Weg,j,k,1) == 1 && rand < p_act 
                            while B(i+a+Weg,j,k,1) == 1 %granule moves through actin
                                a = a + Weg;
                            end
                            if (i+a > x || i+a < 1) && A(i,j,k,2) > ripe %if actin net stretches until end of domain
                                B(i,j,k,1) = 4; %-> then fusion
                            else
                                if B(i+a,j,k,1) == 0 && A(i+a,j,k,1) == 0 %if actin does not stretch until end of domain
                                    B(i+a,j,k,1) = elem_state;
                                    B(i+a,j,k,2) = A(i,j,k,2);
                                    if rand < p_life
                                        B(i+a,j,k,2) = A(i,j,k,2) + 1;
                                    end
                                else
                                    B(i,j,k,1) = elem_state;
                                    B(i,j,k,2) = A(i,j,k,2);
                                    if rand < p_life
                                        B(i,j,k,2) = A(i,j,k,2) + 1;
                                    end
                                end
                            end
                            
                        else %granule does not move through actin and keeps position
                            B(i,j,k,1) = 2; %%
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                        
                    catch
                        if rand < C_calcium && A(i,j,k,2) > ripe
                            B(i,j,k,1) = 4; %fusion
                        else
                            B(i,j,k,1) = 11; %attachment of granule to cell membrane
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                    end
                    continue
                                      
                    
%%%=========== calculation of directed movement along y-axis ===========%%%
                % calculation analogous to movement along x - axis
            elseif elem_state == 7 || elem_state == 8
                    a = 0;
                    % checking if movement in neg. or pos. direction:
                    if elem_state == 7
                        Weg = 1;
                    elseif elem_state == 8
                        Weg = -1;
                    end
                    try
                        if B(i,j+Weg,k,1) == 0 && A(i,j+Weg,k,1) == 0
                            if j+Weg==1 || j+Weg==y
                                B(i,j,k,1) = 11; 
                                B(i,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k,2) = A(i,j,k,2) + 1;
                                end
                              
                            elseif rand < p_5_10 
                                B(i,j+Weg,k,1) = elem_state;
                                B(i,j+Weg,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j+Weg,k,2)= A(i,j,k,2) + 1;
                                end
                            else
                                B(i,j+Weg,k,1) = 2;
                                B(i,j+Weg,k,2) = A(i,j,k,2);
                                if rand < p_life 
                                    B(i,j+Weg,k,2)= A(i,j,k,2) + 1;
                                end
                            end
                            
                        elseif B(i,j+Weg,k,1) == 1 && rand < p_act
                            while B(i,j+a+Weg,k,1) == 1 
                                a = a + 1;
                            end
                            if (j+a > y || j+a < 1) && A(i,j,k,2) > ripe 
                                B(i,j,k,1) = 4;
                            else
                                if B(i,j+a,k,1) == 0 && A(i,j+a,k,1) == 0
                                B(i,j+a,k,1) = elem_state;
                                B(i,j+a,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j+a,k,2)= A(i,j,k,2) + 1;
                                end
                                else
                                    B(i,j,k,1) = elem_state;
                                    B(i,j,k,2) = A(i,j,k,2);
                                    if rand < p_life
                                        B(i,j,k,2) = A(i,j,k,2) + 1;
                                    end
                                end
                            end
                            
                        else 
                            B(i,j,k,1) = 2;
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                        
                    catch
                        if rand < C_calcium && A(i,j,k,2) > ripe
                            B(i,j,k,1) = 4; 
                        else
                            B(i,j,k,1) = 11;
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2)= A(i,j,k,2) + 1;
                            end
                        end
                    end
                    continue
                    
                
%%%=========== calculation of directed movement along z-axis ===========%%%
                % calculation analogous to movement along x - axis
            elseif elem_state == 9 ||elem_state == 10
                    a = 0;
                    % checking if movement in neg. or pos. direction:
                    if elem_state == 9
                        Weg = 1;
                    elseif elem_state == 10
                        Weg = -1;
                    end
                    try
                        if B(i,j,k+Weg,1) == 0 && A(i,j,k+Weg,1) == 0
                            if k+Weg==1 || k+Weg==z
                                B(i,j,k,1) = 11; 
                                B(i,j,k,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k,2) = A(i,j,k,2) + 1;
                                end
                              
                            elseif rand < p_5_10
                                B(i,j,k+Weg,1) = elem_state;
                                B(i,j,k+Weg,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k+Weg,2)= A(i,j,k,2) + 1;
                                end
                            else
                                B(i,j,k+Weg,1) = 2;
                                B(i,j,k+Weg,2) = A(i,j,k,2);
                                if rand < p_life
                                    B(i,j,k+Weg,2)= A(i,j,k,2) + 1;
                                end
                            end
                            
                        elseif B(i,j,k+Weg,1) == 1 && rand < p_act
                            while B(i,j,k+a+Weg,1) == 1 
                                a = a + 1;
                            end
                            if (k+a > z || k+a < 1) && A(i,j,k,2) > ripe  
                                B(i,j,k,1) = 4;
                            else
                                if B(i,j,k+a,1) == 0 && A(i,j,k+a,1) == 0
                                    B(i,j,k+a,1) = elem_state;
                                    B(i,j,k+a,2) = A(i,j,k,2);
                                    if rand < p_life
                                        B(i,j,k+a,2)= A(i,j,k,2) + 1;
                                    end
                                else
                                    B(i,j,k,1) = elem_state;
                                    B(i,j,k,2) = A(i,j,k,2);
                                    if rand < p_life
                                        B(i,j,k,2) = A(i,j,k,2) + 1;
                                    end
                                end
                            end
                            
                        else 
                            B(i,j,k,1) = 2;
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2) = A(i,j,k,2) + 1;
                            end
                        end
                        
                    catch
                        if rand < C_calcium && A(i,j,k,2) > ripe
                            B(i,j,k,1) = 4; 
                        else
                            B(i,j,k,1) = 11;
                            B(i,j,k,2) = A(i,j,k,2);
                            if rand < p_life
                                B(i,j,k,2)= A(i,j,k,2) + 1;
                            end
                        end
                    end
                    continue
                  
                    
%%%======== Movement at cell membrane/ edge of domain ==================%%%    
            elseif elem_state == 11
                    try 
                        if i==1 || i==x
                            a = 0;
                            b = randi([-1 1]);
                            c = randi([-1 1]);
                        elseif j==1 || j==y
                            a = randi([-1 1]);
                            b = 0;
                            c = randi([-1 1]);
                        elseif k==1 || k==z
                            a = randi([-1 1]);
                            b = randi([-1 1]);
                            c = 0;
                        else
                            a = randi([-1 1]);
                            b = randi([-1 1]);
                            c = randi([-1 1]);
                        end
                        
                        if rand < p_life
                            B(i,j,k,2) = A(i,j,k,2) + 1;
                        end
                        
                        if rand < C_calcium && A(i,j,k,2) > ripe   %checking if granule fuses directly
                            B(i,j,k,1) = 4; 
                        else
                            if B(i+a,j+b,k+c,1) == 0 && A(i,j+a,k+b,1) == 0
                                if rand < p_11 
                                    B(i+a,j+b,k+c,1) = 11;
                                else
                                    B(i+a,j+b,k+c,1) = 2;
                                end
                                B(i+a,j+b,k+c,2) = A(i,j,k,2);
                            else
                                B(i,j,k,1) = 11;
                                B(i,j,k,2) = A(i,j,k,2)+0.25;  %increase of ripeness of granules so stuck granules fuse more easily
                            end
                        end
                            
                    catch
                        B(i,j,k,1) = 11;
                        B(i,j,k,2) = A(i,j,k,2);
                    end
                    continue
                                    
            end 
            
        end
    end
end

end                     